function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fs = _interopDefault(require('fs'));
var path = _interopDefault(require('path'));
var chalk = _interopDefault(require('chalk'));

var firstFile;
function cleanStack(str, cwd) {
    if ( cwd === void 0 ) cwd = process.cwd();

    str = str.replace(/^[\s\S]+\n\n([A-Za-z]*Error: )/g, '$1');
    firstFile = null;
    var clean = str.replace(new RegExp(("( |\\()(https?:\\/\\/localhost:\\d+\\/base\\/|" + (cwd.replace(/([\\/[\]()*+$!^.,?])/g, '\\$1')) + "\\/*)?([^\\s():?]*?)(?:\\?[a-zA-Z0-9]+?)?(:\\d+(?::\\d+)?)"), 'g'), replacer);
    if (firstFile != null) {
        var filename = firstFile[0];
        var line = firstFile[1];
        var char = firstFile[2];
        if (line) {
            var read;
            try {
                read = fs.readFileSync(path.resolve(cwd, filename), 'utf8');
            } catch (e) {}
            if (read) {
                var start = Math.max(0, char - 40);
                var startLine = Math.max(0, line - 3);
                read = read.split('\n');
                clean = clean.replace(/\n +/g, '\n  ');
                if (line < read.length) {
                    clean += '\n\n' + chalk.white(highlight(outdent([read.slice(startLine, line).join('\n'),
                        read[line].substr(start, char + 30),new Array(char).join('-') + '^',
                        read.slice(line + 1, line + 4).join('\n')].join('\n'), '    ', process.stdout.columns - 10), line - startLine, 2));
                }
            }
        }
    }
    return chalk.red(clean);
}

function replacer(str, before, root, filename, position) {
    if (firstFile == null) {
        var ref = position.match(/\d+/g) || [];
        var line = ref[0];
        var char = ref[1];
        firstFile = [filename,line - 1,char | 0];
    }
    return before + chalk.blue('./' + filename + chalk.dim(position));
}

function highlight(text, line, count) {
    var lines = text.split('\n');
    return chalk.dim(lines.slice(0, line).join('\n')) + '\n' + lines.slice(line, line + count).join('\n') + '\n' + chalk.dim(lines.slice(line + count).join('\n'));
}

function outdent(str, prefix, width) {
    if ( prefix === void 0 ) prefix = '';
    if ( width === void 0 ) width = 80;

    str = str.replace(/(^\n+|\n+$)/g, '').replace(/\t/, '    ');
    var indents = str.match(/^[ -]+/gm) || [];
    var minLength = indents.reduce(function (indent, value) { return Math.min(indent, value.length); }, indents[0] ? indents[0].length : 0);
    str = str.replace(/^[ -]+/gm, function (str) { return prefix + str.substring(minLength); });
    str = str.replace(/^.*$/gm, function (str) { return str.substring(0, width); });
    return str;
}

function configure(config, layouts) {
    var layout = layouts.colouredLayout;
    return function (logEvent) {
        process.stdout.write(chalk.red(cleanStack(layout(logEvent, config.timezoneOffset))) + '\n');
    };
}

exports.configure = configure;
//# sourceMappingURL=appender.js.map
